import os
import subprocess
import yaml
from kubernetes import client, config

# Load Kubernetes configuration from default location
config.load_kube_config()

# Kubernetes API client
k8s_client = client.ApiClient()

# Docker Hub username (replace with your Docker Hub username)
docker_username = "gsiva333"

def build_and_push_image(function_name):
    """
    Build the Docker image and push it to Docker Hub.
    """
    try:
        # Change directory to function directory
        os.chdir(function_name)
    
        # Build Docker image
        subprocess.run(["docker", "build", "-t", f"{docker_username}/{function_name}:latest", "."], check=True)
    
        # Push Docker image to Docker Hub
        subprocess.run(["docker", "push", f"{docker_username}/{function_name}:latest"], check=True)
    
    except subprocess.CalledProcessError as e:
        print(f"Error occurred while building/pushing Docker image: {e}")

    finally:
        # Change back to original directory
        os.chdir("..")


def create_deployment(function_name):
    """
    Create a Kubernetes Deployment resource for the function.
    """
    try:
        deployment_manifest = {
            "apiVersion": "apps/v1",
            "kind": "Deployment",
            "metadata": {"name": function_name},
            "spec": {
                "replicas": 1,
                "selector": {"matchLabels": {"app": function_name}},
                "template": {
                    "metadata": {"labels": {"app": function_name}},
                    "spec": {
                        "containers": [{
                            "name": function_name,
                            "image": f"{docker_username}/{function_name}:latest",
                            "ports": [{"containerPort": 5000}]  # Assuming function listens on port 8080
                        }]
                    }
                }
            }
        }

        #yourusername Create Deployment in Kubernetes
        api_instance = client.AppsV1Api(k8s_client)
        api_instance.create_namespaced_deployment(namespace="default", body=deployment_manifest)
        print(f"Deployment created for {function_name}.")
    except client.rest.ApiException as e:
        print(f"Error occurred while creating Deployment: {e}")

def create_service(function_name):
    """
    Create a Kubernetes Service resource for the function.
    """
    try:
        service_manifest = {
            "apiVersion": "v1",
            "kind": "Service",
            "metadata": {"name": f"{function_name}-service"},
            "spec": {
                "selector": {"app": function_name},
                "ports": [{"protocol": "TCP", "port": 8080, "targetPort": 5000}],  # Assuming function listens on port 8080
                "type": "LoadBalancer"  # Expose service only within the cluster
            }
        }

        # Create Service in Kubernetes
        #k8s_client.create_namespaced_service(namespace="default", body=service_manifest)
        api_instance = client.CoreV1Api()
        api_instance.create_namespaced_service(namespace="default", body=service_manifest)
        print(f"Service created for {function_name}.")
    except client.rest.ApiException as e:
        print(f"Error occurred while creating Service: {e}")

"""
def get_external_ip(service_name):
   
    try:
        # Wait for the external IP to be assigned
        while True:
            service = api_instance.read_namespaced_service(name=service_name, namespace="default")
            if service.status.load_balancer.ingress:
                external_ip = service.status.load_balancer.ingress[0].ip
                if external_ip:
                    return external_ip
            time.sleep(5)  # Wait for 5 seconds before checking again
    except client.rest.ApiException as e:
        print(f"Error occurred while getting external IP: {e}")
"""
def deploy_function(function_name):
    """
    Deploy the function to Kubernetes.
    """
    try:
        build_and_push_image(function_name)
        create_deployment(function_name)
        create_service(function_name)

        # Wait for the external IP to be assigned
        #external_ip = get_external_ip(f"{function_name}-service")
        #print(f"External IP for {function_name}-service: {external_ip}")

        
    except Exception as e:
        print(f"Error occurred during deployment of {function_name}: {e}")

if __name__ == "__main__":
    function_name = input("Enter the name of the function directory: ")
    deploy_function(function_name)

